<?php

/**
 * This class controls the UI side of the Checkout page
 */
class Montonio_Payments_Checkout
{
    protected $_payment_handle_style;
    protected $_payment_handle_extra_css;
    protected $_payment_logos_list;
    protected $_preferred_country;
    protected $_is_non_default_description = false;
    protected $_attempt_to_select_payment = false;
    protected $regions = [
        'EE' => 'Estonia',
        'LT' => 'Lithuania',
        'LV' => 'Latvia',
        'FI' => 'Finland',
        'PL' => 'Poland',
        'DE' => 'Germany'
     ];

    /**
     * The description shown at checkout
     *
     * @var string
     */
    protected $_description;

    /**
     * Banks to show at checkout
     * (configured in Montonio_Payments_Settings)
     *
     * @var Object
     */
    protected $_bankList;

    public function get_description_html()
    {

        $idCurrency      = Context::getContext()->cart->id_currency;
        $paymentCurrency = new Currency($idCurrency); // should contain iso_code

        if (empty($this->_bankList) || !isset($paymentCurrency->iso_code)) {
            return $this->get_default_description();
        } else {

            $setupData = json_decode($this->_bankList, true);
            $bankList = $this->getFilteredBankList($setupData, $this->get_preferred_country(), (int)Configuration::get('MONTONIO_PAYMENTS_HIDE_COUNTRY'));
            $regions = array_keys($bankList['paymentMethods']['paymentInitiation']['setup']);

            return $this->get_attempt_to_select_payment_html() . '
                <div id="montonio-checkout-1"></div>
                <script>
                var selectMontonioAspsp = function() {
                       $(\'#montonio_payments_preselected_aspsp\').val(this.getPreferredProvider());
                }
                var selectMontonioCountry = function() {
                       $(\'#montonio_payments_preselected_country\').val(this.getPreferredRegion());
                }
                window.onMontonioLoaded = function() {
                       let checkout =  Montonio.Checkout.PaymentInitiation.create({
                            // The JSON string obtained from the HTTP request GET /stores/setup.
                            storeSetupData: \'' . json_encode($bankList) . '\' ,
                            currency: \'' . $paymentCurrency->iso_code . '\',
                            targetId: \'montonio-checkout-1\',
                            defaultRegion: \'' . $this->get_preferred_country() . '\',
                            shouldAllowDeselection: true,
                            regionInputName: "preselectedCountry",
                            inputName: "preselectedAspsp",
                            description: \'' . $this->get_default_description() . '\',
                            onProviderClick: window.selectMontonioAspsp ,
                            onRegionSelectorChange: window.selectMontonioCountry ,
                            regions: '. json_encode($regions).'  ,
                        });
                       checkout.init();
                       setTimeout(function(checkout) {
                              window.selectMontonioAspsp.bind(checkout)();
                               window.selectMontonioCountry.bind(checkout)();
                           }, 400, checkout);
                       };
                </script>
            ';
        }
    }

    /**
     * @param array $bankList
     * @param string $defaultCountry
     * @param int $hideCountry
     * @return array
     */
    public function getFilteredBankList(array $bankList, string $defaultCountry, int $hideCountry = 0) {

        if ($hideCountry == '1') {
            $bankList['paymentMethods']['paymentInitiation']['setup'] = array_filter(
                $bankList['paymentMethods']['paymentInitiation']['setup'],
                function ($var) use ($defaultCountry) {
                    return $var == $defaultCountry;
                },
                ARRAY_FILTER_USE_KEY
            );
        }
        return $bankList;
    }

    // =========================================================================
    // Checkout HTML types
    // =========================================================================

    protected function get_default_description()
    {
        return $this->get_description(); // TODO: Translations
    }

    protected function get_attempt_to_select_payment_html() {
        return '<input type="hidden" id="montonio_attempt_to_select_payment_method" value="'. ($this->get_attempt_to_select_payment() ? 'yes' : 'no') .'">';
    }

    // =========================================================================
    // Getters and setters
    // =========================================================================

    /**
     * @param object $payment_handle_style
     */
    public function set_payment_handle_style($payment_handle_style)
    {
        //@TODO refactor this
        $this->_payment_handle_style = $payment_handle_style;
    }

    /**
     * JSON-decode bankList
     *
     * @param object $payment_handle_style
     */
    public function set_banklist($banklistJson)
    {
        $this->_bankList = $banklistJson;
    }

    public function set_description($description) {
        $this->_description = $description;
    }

    public function get_description() {
        return $this->_description;
    }

    public function set_preferred_country($country) {
        $this->_preferred_country = $country;
    }

    public function get_preferred_country() {
        return $this->_preferred_country;
    }

    public function set_regions($regions) {
        $this->regions = $regions;
    }

    public function get_regions() {
        return $this->regions;
    }

    public function set_attempt_to_select_payment($attempt_to_select_payment) {
        $this->_attempt_to_select_payment = $attempt_to_select_payment == 'yes';
    }

    public function get_attempt_to_select_payment() {
        return $this->_attempt_to_select_payment;
    }
}